'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _d = require('d3');

var _d2 = _interopRequireDefault(_d);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/*
 * things => array of things to be represented by bars (individual rectangles)
 * valFunc => func(thing, i) returning a number that gives relative height of bar
 *
 * optional functions to call if a bar is hovered over:
 *    highlight, endHighlight => func(thing,passthrough,i,mouseEvent)
 * optional function to determine if a bar should be highlighted:
 *    isHighlighted => func(thing,passthrough,i)
 * 
 * sortBy => func(thing) should return value to determine bar order
 *           defaults to bar value
 *          
 */

var SparkBarsChart = function (_Component) {
  _inherits(SparkBarsChart, _Component);

  function SparkBarsChart() {
    _classCallCheck(this, SparkBarsChart);

    return _possibleConstructorReturn(this, Object.getPrototypeOf(SparkBarsChart).apply(this, arguments));
  }

  _createClass(SparkBarsChart, [{
    key: 'shouldComponentUpdate',
    value: function shouldComponentUpdate(nextProps, nextState) {
      return this.props.things != nextProps.things || this.props.highlighted != nextProps.highlighted;
    }
  }, {
    key: 'render',
    value: function render() {
      var _props = this.props;
      var passthrough = _props.passthrough;
      var width = _props.width;
      var height = _props.height;
      var isHighlighted = _props.isHighlighted;
      var highlight = _props.highlight;
      var endHighlight = _props.endHighlight;

      var valFunc = this.props.valFunc || function (d) {
        return d;
      };
      var things = _lodash2.default.sortBy(this.props.things, this.props.sortBy || function (d) {
        return d;
      });
      var ext = _d2.default.extent(things.map(function (thing, i) {
        return valFunc(thing, i);
      }));
      var dumbExt = [0, ext[1]];
      var yscale = _d2.default.scale.linear().domain(dumbExt).range([0, height]);
      var barWidth = width / things.length;
      var bars = [];
      var self = this;
      things.forEach(function (thing, i) {
        bars.push(_react2.default.createElement(SparkBarsBar, {
          passthrough: passthrough,
          thing: thing,
          valFunc: valFunc,
          isHighlighted: isHighlighted,
          highlight: highlight,
          endHighlight: endHighlight,
          x: barWidth * i,
          chartHeight: self.props.height,
          barWidth: barWidth,
          yscale: yscale,
          i: i,
          key: i
        }));
      });
      return _react2.default.createElement(
        'div',
        null,
        _react2.default.createElement(
          'svg',
          { width: width, height: height },
          bars
        )
      );
    }
  }]);

  return SparkBarsChart;
}(_react.Component);

SparkBarsChart.propTypes = {
  things: _react.PropTypes.array.isRequired,
  width: _react.PropTypes.number.isRequired,
  height: _react.PropTypes.number.isRequired
};
exports.default = SparkBarsChart;

var SparkBarsBar = function (_Component2) {
  _inherits(SparkBarsBar, _Component2);

  function SparkBarsBar() {
    _classCallCheck(this, SparkBarsBar);

    return _possibleConstructorReturn(this, Object.getPrototypeOf(SparkBarsBar).apply(this, arguments));
  }

  _createClass(SparkBarsBar, [{
    key: 'render',
    value: function render() {
      var _props2 = this.props;
      var passthrough = _props2.passthrough;
      var thing = _props2.thing;
      var valFunc = _props2.valFunc;
      var i = _props2.i;
      var yscale = _props2.yscale;
      var chartHeight = _props2.chartHeight;
      var x = _props2.x;
      var barWidth = _props2.barWidth;
      var highlight = _props2.highlight;
      var isHighlighted = _props2.isHighlighted;
      var endHighlight = _props2.endHighlight;

      var height = yscale(valFunc(thing, i));
      var y = chartHeight - height;
      highlight = highlight || _lodash2.default.noop;
      isHighlighted = isHighlighted || _lodash2.default.noop;
      endHighlight = endHighlight || _lodash2.default.noop;
      var highlighted = isHighlighted(thing, passthrough, i);
      return _react2.default.createElement(
        'g',
        { transform: "translate(" + x + ")" },
        _react2.default.createElement('rect', {
          style: barStyles['background' + !!highlighted],
          width: barWidth,
          height: chartHeight,
          onMouseOver: function onMouseOver(evt) {
            return highlight(thing, passthrough, i, evt);
          },
          onMouseOut: function onMouseOut(evt) {
            return endHighlight(thing, passthrough, i, evt);
          }
        }),
        _react2.default.createElement('rect', { y: chartHeight - height,
          style: barStyles['normal' + !!highlighted],
          width: barWidth,
          height: height,
          onMouseOver: function onMouseOver(evt) {
            return highlight(thing, passthrough, i, evt);
          },
          onMouseOut: function onMouseOut(evt) {
            return endHighlight(thing, passthrough, i, evt);
          }
        })
      );
    }
  }]);

  return SparkBarsBar;
}(_react.Component);

;
function barStyle(type, highlighted) {
  var opacities = {
    background: .2,
    normal: 1
  };
  var colors = {
    background: 'steelblue',
    normal: 'steelblue'
  };
  return {
    fill: colors[type],
    strokeWidth: 1,
    stroke: highlighted ? 'steelblue' : 'white',
    opacity: opacities[type] / (highlighted ? 1 : 2)
  };
}
var barStyles = _lodash2.default.chain([true, false]).map(function (highlighted) {
  return _lodash2.default.map(['normal', 'background'], function (type) {
    return [type + highlighted, barStyle(type, highlighted)];
  });
}).flatten().object().value();